//
//  MainViewController.m
//  Komakusa
//
//  Created by ユウスケ on 2012/09/14.
//  Copyright (c) 2012年 arms22. All rights reserved.
//

#import "MainViewController.h"
#import "AudioSignalAnalyzer.h"
#import "FSKSerialGenerator.h"
#import "FSKRecognizer.h"
#import "SensorSerialProtocol.h"

@interface MainViewController ()

@end

@implementation MainViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    _tempLabel.layer.cornerRadius = 5;
    _tempLabel.clipsToBounds = true;
    
    _humidityLabel.layer.cornerRadius = 5;
    _humidityLabel.clipsToBounds = true;
    
    _dewPointLabel.layer.cornerRadius = 5;
    _dewPointLabel.clipsToBounds = true;
    
    _sspState = 0;
    
    // Audio Sessionの初期化
	AVAudioSession *session = [AVAudioSession sharedInstance];
	session.delegate = self;
	if(session.inputIsAvailable){
		[session setCategory:AVAudioSessionCategoryPlayAndRecord error:nil];
	}else{
		[session setCategory:AVAudioSessionCategoryPlayback error:nil];
	}
	[session setActive:YES error:nil];
	[session setPreferredIOBufferDuration:0.023220 error:nil];
	
    // FSK復調モジュール初期化
	_recognizer = [[FSKRecognizer alloc] init];
	[_recognizer addReceiver:self];
    
	_analyzer = [[AudioSignalAnalyzer alloc] init];
	[_analyzer addRecognizer:_recognizer];
	
    // FSK変調モジュール初期化
	_generator = [[FSKSerialGenerator alloc] init];
	[_generator play];
	
    // 録音開始
	if(session.inputIsAvailable){
		[_analyzer record];
	}
    
    // データリクエストタイマー起動
    [NSTimer scheduledTimerWithTimeInterval:5.0
                                     target:self
                                   selector:@selector(intervalReadReq:)
                                   userInfo:nil
                                    repeats:YES];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    } else {
        return YES;
    }
}
#pragma mark - Audio Session

- (void)restartAnalyzerAndGenerator:(BOOL)isInputAvailable
{
	AVAudioSession *session = [AVAudioSession sharedInstance];
	[session setActive:YES error:nil];
	[_analyzer stop];
	[_generator stop];
	if(isInputAvailable){
		[session setCategory:AVAudioSessionCategoryPlayAndRecord error:nil];
		[_analyzer record];
	}else{
		[session setCategory:AVAudioSessionCategoryPlayback error:nil];
	}
	[_generator play];
}

- (void)inputIsAvailableChanged:(BOOL)isInputAvailable
{
	NSLog(@"inputIsAvailableChanged %d",isInputAvailable);
	[self restartAnalyzerAndGenerator:isInputAvailable];
}

- (void)beginInterruption
{
	NSLog(@"beginInterruption");
}

- (void)endInterruption
{
	NSLog(@"endInterruption");
}

#pragma mark - FSK

- (void)intervalReadReq:(NSTimer*)theTimer
{
    // データリクエスト送信
    [_generator writeByte: (UInt8)SSP_READ];
    // 状態変数初期化
    _sspState = SSP_READ;
    _sspRecvDataLength = 0;
}

#define D1 (-40.1)
#define D2 (0.01)
#define C1 (-2.0468)
#define C2 (0.0367)
#define C3 (-1.5955E-6)
#define T1 (0.01)
#define T2 (0.00008)

- (void) receivedChar:(char)input
{
    switch (_sspState) {
        case SSP_READ:
            //受信バッファにデータを格納
            _sspRecvData[_sspRecvDataLength++] = input;
            //データ受信完了？
            if(_sspRecvDataLength >= (SSP_DATA_LEN)){
                //CRC値計算
                UInt8 sum = crc8(_sspRecvData, SSP_DATA_LEN);
                //一致したか？
                if(sum == 0){
                    float temp, humi, dewpoint;
                    UInt16 tempRaw, humiRaw;
                    
                    tempRaw = _sspRecvData[0] << 8 | _sspRecvData[1];
                    humiRaw = _sspRecvData[2] << 8 | _sspRecvData[3];
                    
                    //温度計算
                    temp = D1 + D2 * (float) tempRaw;
                    
                    //湿度計算
                    humi = C1 + C2 * humiRaw + C3 * humiRaw * humiRaw;
                    humi = (temp - 25.0) * (T1 + T2 * humiRaw) + humi;
                    if (humi > 100.0) humi = 100.0;
                    if (humi < 0.1) humi = 0.1;
                    
                    //露点温度計算
                    dewpoint = log(humi/100) + (17.62 * temp) / (243.12 + temp);
                    dewpoint = 243.12 * dewpoint / (17.62 - dewpoint);
                    
                    //表示更新
                    _tempLabel.text = [NSString stringWithFormat:@"%.1f°", temp];
                    _humidityLabel.text = [NSString stringWithFormat:@"%.1f%%", humi];
                    _dewPointLabel.text = [NSString stringWithFormat:@"%.1f°", dewpoint];
                }
                _sspState = 0;
                _sspRecvDataLength = 0;
            }
            break;
        default:
            _sspState = 0;
            _sspRecvDataLength = 0;
            break;
    }
}

#pragma mark - Flipside View Controller

- (void)flipsideViewControllerDidFinish:(FlipsideViewController *)controller
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        [self dismissModalViewControllerAnimated:YES];
    } else {
        [self.flipsidePopoverController dismissPopoverAnimated:YES];
    }
}

- (void)dealloc
{
    [_flipsidePopoverController release];
    [super dealloc];
}

- (IBAction)showInfo:(id)sender
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        FlipsideViewController *controller = [[[FlipsideViewController alloc] initWithNibName:@"FlipsideViewController" bundle:nil] autorelease];
        controller.delegate = self;
        controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
        [self presentModalViewController:controller animated:YES];
    } else {
        if (!self.flipsidePopoverController) {
            FlipsideViewController *controller = [[[FlipsideViewController alloc] initWithNibName:@"FlipsideViewController" bundle:nil] autorelease];
            controller.delegate = self;
            
            self.flipsidePopoverController = [[[UIPopoverController alloc] initWithContentViewController:controller] autorelease];
        }
        if ([self.flipsidePopoverController isPopoverVisible]) {
            [self.flipsidePopoverController dismissPopoverAnimated:YES];
        } else {
            [self.flipsidePopoverController presentPopoverFromBarButtonItem:sender permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
        }
    }
}

@end
