#include <SoftModem.h>
#include <Sensirion.h>
#include "SensorSerialProtocol.h"

// SotfModemインスタンス生成
SoftModem myModem;

// SHT11接続端子定義
const uint8_t DATA_PIN  =  8;
const uint8_t CLOCK_PIN =  9;

// SHT11インスタンス生成
Sensirion tempSensor = Sensirion(DATA_PIN, CLOCK_PIN);

// 生温度データ
uint16_t temperatureRaw = 0;
// 生湿度データ
uint16_t humidityRaw = 0;
// 送信用バッファ
uint8_t data[5];

void setup()
{
  // SoftModem受信開始
  myModem.begin();
}

void loop()
{
  // データ受信したか？
  if(myModem.available()){
    // 1byte読み出す
    int c = myModem.read();
    switch(c){
    case SSP_READ:
      // 温度・湿度計測
      tempSensor.measTemp(&temperatureRaw);
      tempSensor.measHumi(&humidityRaw);
      // 送信用バッファにデータを格納し、CRC8を計算
      data[0] = temperatureRaw >> 8;
      data[1] = temperatureRaw & 0xff;
      data[2] = humidityRaw >> 8;
      data[3] = humidityRaw & 0xff;
      data[4] = crc8(data, 4);
      // 5byte送信
      myModem.write(data, 5);
      break;
    default:
      break;
    }
  }
  else{
    // ポーリングの間隔を適度にあける
    delay(20);
  }
}

